window.initCanvasParticles = function () {
	let target = document.getElementsByClassName('canvas_particles')[0],
		canvas = document.createElement('canvas'),
		totalWidth = target.offsetWidth,
		totalHeight = target.offsetHeight,
		dataset = target.dataset;
	canvas.setAttribute('width', target.offsetWidth);
	canvas.setAttribute('height', target.offsetHeight);
	canvas.style.position='absolute';
	canvas.style.top='0px';
	target.insertBefore(canvas,target.childNodes[0]);
	let ctx = canvas.getContext('2d');

	let config = {
		startx: dataset.startx || 0,
		starty: dataset.starty || 0,
		bgColor: dataset.bgColor || '#000',
		lineRange: 100,
		lineColor: '#fff',
		step: {
			x: dataset.stepx || .21,
			y: dataset.stepy || .5,
			r: dataset.stepr || .01
		},
		r: dataset.r || 5,
		ballCount: dataset.ballCount || parseInt(totalHeight * totalWidth / 5000),
		maxColor: parseInt('ffffff', 16)
	};

	let utils = {
		drawLine (start, end,distance) {
			ctx.beginPath();
			ctx.moveTo(...start);
			ctx.globalAlpha = 1-(distance / config.lineRange);
			ctx.strokeStyle=config.lineColor;
			ctx.lineTo(...end);
			ctx.stroke();
			ctx.globalAlpha = 1;
			ctx.closePath();
		},
		deleteBall (ballId) {
			delete globalStorage.balls[ballId];
			globalStorage.ballIds.splice(globalStorage.ballIds.indexOf(ballId), 1);
		},
		updateBall (newBall) {
			globalStorage.balls[newBall.id] = newBall;
		},
		toRadian (angle) {
			return angle / 180 * Math.PI;
		},
		toAngle (radian) {
			return radian / Math.PI * 180;
		},
		getRandomColor () {
			let colorArr = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 'A', 'B', 'C', 'D', 'E', 'F'],
				color = '';
			for (let i = 0; i < 6; i++) {
				let randomIndex = Math.floor(Math.random() * (colorArr.length - 1));
				color += colorArr[randomIndex];
			}
			return color;
		},
		changeColor (ball) {
			let maxInTen = config.maxColor,
				minInTen = 0,
				colorInTen = parseInt(ball.color.inHax, 16);
			let toColorInTen = colorInTen + ball.color.direction;
			if (toColorInTen > maxInTen || toColorInTen < minInTen) {
				ball.color.direction = -ball.color.direction;
				toColorInTen = colorInTen + ball.color.direction;
			}
			ball.color.inHax = toColorInTen.toString(16);
			this.updateBall(ball);
		},
		knockedCheck (ball) {
			//检查是否到达边界。
			function reverse (xOrY) {
				xOrY === 'x' ? ball.direction.x = -ball.direction.x : ball.direction.y = -ball.direction.y;
			}

			if (ball.prevPostion.x < 0 || ball.prevPostion.x > totalWidth) {
				reverse('x');
			} else if (ball.prevPostion.y < 0 || ball.prevPostion.y > totalHeight) {
				reverse('y');
			}
			utils.updateBall(ball);
		},
		generateId () {
			let id = parseInt(Math.random() * 10000);
			if (globalStorage.ballIds.includes(id)) {
				id = utils.generateId();
			} else {
				globalStorage.ballIds.push(id);
				return id;
			}
		},
		checkAndDrawLine (ball) {
			let x = ball.prevPostion.x, y = ball.prevPostion.y, range = config.lineRange;
			//检查所有的点是否在范围内，在的话连线
			for (let key in globalStorage.balls) {
				if (globalStorage.balls.hasOwnProperty(key)) {
					let anotherBall = globalStorage.balls[key],
						anotherX = anotherBall.prevPostion.x,
						anotherY = anotherBall.prevPostion.y;
					let distance = Math.sqrt(Math.pow(anotherX - x,2) + Math.pow(anotherY - y,2));
					if (distance < range) {
						utils.drawLine([x,y],[anotherX,anotherY],distance);
					}
				}
			}
		}
	};

	let globalStorage = {
		ballIds: [],
		balls: {}
	};

	function createBall () {
		//个体
		let ball = {
			prevPostion: {
				x: Math.random() * totalWidth,
				y: Math.random() * totalHeight
			},
			direction: { //方向
				x: Math.random() > 0.5 ? 1 : -1,
				y: Math.random() > 0.5 ? 1 : -1,
				r: 1
			},
			color: {
				inHax: utils.getRandomColor(),
				direction: -1
			},
			r: Math.random() * config.r,
			id: utils.generateId()
		};
		globalStorage.balls[ball.id] = ball;
	}

	function changeBallStatus (ball) {
		//更新方向
		utils.knockedCheck(ball);
		//更新位置
		ball.prevPostion.x = ball.prevPostion.x + config.step.x * ball.direction.x;
		ball.prevPostion.y = ball.prevPostion.y + config.step.y * ball.direction.y;
		//更新大小
		let rRandom = Math.random();
		let changedR = ball.r - (config.step.r * ball.direction.r);
		//更新颜色
		utils.changeColor(ball);
		//检查是否已经到极限了
		if (changedR < 0 || changedR > config.r) {
			//到极限了就反方向增长
			ball.direction.r = -ball.direction.r;
			//改了方向重新计算
			ball.r = ball.r - (config.step.r * ball.direction.r);
		} else {
			ball.r = changedR;
		}

		utils.updateBall(ball);
	}



	function moveBall (ball) {
		ctx.beginPath();
		ctx.fillStyle = `#${ball.color.inHax}`;
		ctx.arc(ball.prevPostion.x, ball.prevPostion.y, ball.r, 0, utils.toRadian(360));
		ctx.fill();
		ctx.closePath();
		//绘制连线
		utils.checkAndDrawLine(ball);
		changeBallStatus(ball);
	}

	function gogogo () {
		//清空画布
		ctx.fillStyle = config.bgColor;
		ctx.fillRect(0, 0, totalWidth, totalHeight);
		//获取所有该画的球,移动一步
		for (let key in globalStorage.balls) {
			if (globalStorage.balls.hasOwnProperty(key)) {
				moveBall(globalStorage.balls[key]);
			}
		}
		requestAnimationFrame(gogogo);
	}

	function init () {
		//先创建出一堆球
		for (let i = 0; i < config.ballCount; i++) {
			createBall();
		}
		//运行
		gogogo();
	}

	init();
};